package fr.username404.snowygui.gui.feature

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.ClickGui
import fr.username404.snowygui.Snowy
import fr.username404.snowygui.config.Configuration
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.FontUtil
import fr.username404.snowygui.gui.Renderable.Rendering.defaultRectFunc
import fr.username404.snowygui.gui.elements.ClickBox
import fr.username404.snowygui.gui.feature.ButtonInfo.Companion.Type
import kotlin.reflect.full.findAnnotation

sealed class ButtonImpl: ColoredElement(0.0, 0.0, 73, 8, opacity = 0.60F) {
    companion object {
        var done: Boolean = false; private set
        fun initialize() {
            done = true
            Snowy.annotatedButtons.forEach { // Initializes every button
                ((try {
                    it.getConstructor().newInstance()
                } catch (e: NoSuchMethodException) {
                    it.getDeclaredField("INSTANCE").get(null)
                }) as ButtonImpl).let { impl ->
                    with(impl) {
                        if (info.kind == ButtonInfo.Companion.Type.TOGGLE) {
                            Configuration.enabledFeatures[title]?.let { bool ->
                                toggled = bool
                            }
                        }
                    }
                    ClickGui.components.filterIsInstance<ClickBox>().find { box ->
                        box.name!!.key == impl.info.parent.translationKey
                    }?.buttons!!.add(impl)
                }
            }
        }
    }
    private val info = this::class.findAnnotation<ButtonInfo>() ?: throw Exception("Missing @Button annotaton")
    override var color = info.parent.categoryColor
    val title = this@ButtonImpl::class.simpleName
    protected open fun execAction() = Unit
    private var wasWithinBounds: Boolean = false
    var toggled: Boolean = false; private set(value) {
        if (value) lightUp() else if (field) lightDown()
        if ((!(!field && !value))) {
            field = value
            execAction()
        } else field = value
    }
    private fun lightUp() { opacity += ButtonInfo.lightningFactor }
    private fun lightDown() { opacity -= ButtonInfo.lightningFactor }
    override fun mouseClicked(d: Double, e: Double, i: Int): Boolean {
        wasWithinBounds = withinBounds(d, e).also {
            if (it && (info.kind == Type.TOGGLE)) {
                toggled = !toggled
            }
        }
        return false
    }

    override fun mouseReleased(d: Double, e: Double, i: Int): Boolean {
        if (wasWithinBounds && (info.kind == Type.CLICK)) {
            toggled = true
        }; return false
    }
    override fun render(poseStack: PoseStack?) {
        RenderSystem.disableTexture()
        RenderSystem.enableBlend()
        defaultRectFunc()
        RenderSystem.enableTexture()
        RenderSystem.disableBlend()
        if (poseStack != null && title != null) {
            FontUtil.drawScaled(poseStack, title, x + 1, y + 1, 0.75F)
        }
    }
}
